from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class CompanyModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_company_by_id(self, company_id: int) -> list[dict[str, Any]]:
        response = self.read("res.company", [company_id])
        return response

    def get_company_id_by_name(self, company_name: str) -> int:
        response = self.search("res.company", [[["name", "=", company_name]]])
        if response:
            return response[0]
        raise Exception(f"Company '{company_name}' not found")



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class PartnerCategoryModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_category_by_id(self, category_id: int) -> dict[str, Any]:
        response = self.read("res.partner.category", [category_id])
        if response:
            return response[0]
        raise Exception(f"Category '{category_id}' not found")

    def get_category_id_by_name(self, category_name: str) -> int:
        response = self.search("res.partner.category", [[["name", "=", category_name]]])
        if response:
            return response[0]
        raise Exception(f"Category '{category_name}' not found")



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class UserModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_user_by_id(
        self, user_id: int, fields: list[str] | None = None
    ) -> dict[str, Any]:
        response = self.read("res.users", [user_id], fields)
        if response:
            return response[0]
        raise Exception(f"User '{user_id}' not found")

    def get_user_id_by_name(self, username: str) -> int:
        response = self.search("res.users", [[["name", "=", username]]])
        if response:
            return response[0]
        raise Exception(f"User '{username}' not found")



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class PartnerModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_partner_by_id(
        self, partner_id: int, fields: list[str] | None = None
    ) -> list[dict[str, Any]]:
        response = self.read("res.partner", [partner_id], fields)
        return response

    def get_partner_id_by_name(
        self, partner_name: str, fields: list[str] | None = None
    ) -> int:
        response = self.search("res.partner", [[["name", "=", partner_name]]], fields)
        if response:
            return response[0]
        raise Exception(f"Partner '{partner_name}' not found")

    def update_partner(
        self,
        partner_id: int,
        partner_data: dict[str, Any],
    ) -> bool:
        response = self.update("res.partner", partner_id, partner_data)
        return response

    def create_partner_from_scratch(
        self,
        partner_data: dict[str, Any],
    ) -> int | list[int]:
        response = self.create("res.partner", [partner_data])
        return response

    def create_partner(
        self,
        name: str,
        street: str,
        city: str,
        state_id: int,
        country_id: int,
        zip_code: str,
        nif: str,
        contact_type: str,
        anf_code: str | None,
        pharmacy_name: str | None,
        owner_name: str | None,
        phone: str | None,
        mobile: str | None,
        email: str | None,
        list_category_id: list[int],
        list_crm_tag_id: list[int],
        vendor_user_id: int | None,
        reference: str | None,
        language: str = "pt_PT",
        is_company: bool = True,
        company_type: str = "company",
    ) -> int | list[int]:
        partner_data = {
            "name": name,
            "street": street,
            "city": city,
            "state_id": state_id,  # "Lisboa", # aqui deve-se buscar o state_id pelo nome do estado
            "country_id": country_id,  # "Portugal", # aqui deve-se buscar o country_id pelo nome do país
            "zip": zip_code,
            "vat": nif,  # NIF
            "x_studio_tipo_de_contacto_1": contact_type,  # Tipo de Contacto
            "x_studio_cdigo_anf_1": anf_code,  # apenas para tipo farmácia
            "x_studio_nome_da_farmcia_1": pharmacy_name,  # apenas para tipo farmácia
            "x_studio_nome_do_proprietrio_1": owner_name,  # apenas para tipo farmácia
            "phone": phone,
            "mobile": mobile,
            "email": email,
            "lang": language,  # idioma
            "category_id": list_category_id,  # Etiquetas / labels - models: res.partner.category - fields: name, active (booleean), parent_id (categoria pai), display_name
            "x_studio_etiquetas_crm": list_crm_tag_id,  # model: crm.tag - fields: name, id, color
            "user_id": vendor_user_id,  # models: res.users - fields: name
            "ref": reference,  # Referência
            "is_company": is_company,  # se é empresa ou não
            "company_type": company_type,  # tipo de empresa
        }
        response = self.create("res.partner", [partner_data])
        return response

    def get_and_read_all_partners(
        self, fields: list[str] | None = None, limit: int = 10, offset: int = 0
    ) -> list[dict[str, Any]]:
        response = self.search_read("res.partner", [], fields or [], limit, offset)
        return response

    def count_partners_by_any_filter(self, filter: list[Any] | None = None) -> int:
        response = self.search_count("res.partner", [filter or []])
        return response

    def get_and_read_partners_by_any_filter(
        self,
        filter: list[Any],
        fields: list[str] | None = None,
        limit: int = 10,
        offset: int = 0,
    ) -> list[dict[str, Any]]:
        response = self.search_read(
            "res.partner", [filter], fields or [], limit, offset
        )
        return response

    def action_archive_partner(self, partner_id: int) -> dict[str, Any]:
        response = self.execute_action("res.partner", "action_archive", partner_id)
        return response



from ._account_tax import AccountTaxModel as AccountTaxModel
from ._country import CountryModel as CountryModel
from ._crm_tag import CRMTagModel as CRMTagModel
from ._integration import OdooIntegration as OdooIntegration
from ._partner_category import PartnerCategoryModel as PartnerCategoryModel
from ._partner import PartnerModel as PartnerModel
from ._product_category import ProductCategoryModel as ProductCategoryModel
from ._product_template import ProductTemplateModel as ProductTemplateModel
from ._product import ProductModel as ProductModel
from ._stock import StockModel as StockModel
from ._sale_order import SaleOrderModel as SaleOrderModel
from ._user import UserModel as UserModel
from ._analytic_account import AnalyticAccountModel as AnalyticAccountModel
from ._payment_term import PaymentTermModel as PaymentTermModel
from ._company import CompanyModel as CompanyModel
from ._purchase_order import PurchaseOrderModel as PurchaseOrderModel
from ._account_move import AccountMoveModel as AccountMoveModel
from ._account_invoice import AccountInvoiceModel as AccountInvoiceModel
from ._mail_message import MailMessageModel as MailMessageModel
from ._ir_attachment import IRAttachmentModel as IRAttachmentModel
from ._account_journal import AccountJournalModel as AccountJournalModel
from ._account_account import AccountAccountModel as AccountAccountModel
from ._generic import OdooGenericIntegration as OdooGenericIntegration



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration
from .exceptions import LotProductNotFoundError, TooManyLotProducError


class StockModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_product_lot_id_by_serie_name(self, serie_name: str) -> int:
        response = self.search("stock.production.lot", [[["name", "=", serie_name]]])
        if response:
            return response[0]
        raise Exception(f"Product Lot '{serie_name}' not found")

    def get_product_lot_id_by_serie_name_and_product_id(
        self, serie_name: str, product_id: int, company_id: int | None = None
    ) -> int:
        if company_id:
            search_params = [
                [
                    ["name", "=", serie_name],
                    ["product_id", "=", product_id],
                    ["company_id", "=", company_id],
                ]
            ]
        else:
            search_params = [
                [["name", "=", serie_name], ["product_id", "=", product_id]]
            ]
        response = self.search("stock.production.lot", search_params)
        if len(response) > 1:
            raise TooManyLotProducError(
                f"Multiple Product Lots with name '{serie_name}' found"
            )
        if response:
            return response[0]
        raise LotProductNotFoundError(f"Product Lot '{serie_name}' not found")

    def get_product_lot_by_id(self, product_lot_id: int) -> list[dict[str, Any]]:
        response = self.read("stock.production.lot", [product_lot_id])
        return response

    def get_warehouse_id_by_name(self, warehouse_name: str, company_id: int) -> int:
        response = self.search(
            "stock.warehouse",
            [[["name", "=", warehouse_name], ["company_id", "=", company_id]]],
        )
        if response:
            return response[0]
        raise Exception(f"Warehouse '{warehouse_name}' not found")

    def get_warehouse_by_id(self, warehouse_id: int) -> list[dict[str, Any]]:
        response = self.read("stock.warehouse", [warehouse_id])
        return response

    def create_product_lot(
        self,
        product_id: int,
        serie_number: str,
        company_id: int,
        expiration_date: str,
        removal_date: str,
        alert_date: str,
        use_date: str,
        product_uom_id: int = 1,
    ) -> int | list[int]:
        product_lot = {
            "product_id": product_id,
            "name": serie_number,
            "company_id": company_id,
            # "product_qty": product_qty,
            "product_uom_id": product_uom_id,  # unidade de medida - 1 é unidade
            "expiration_date": expiration_date,  # Data de Validade
            "removal_date": removal_date,  # Data de Remoção
            "alert_date": alert_date,  # Data de Alerta
            "use_date": use_date,  # Consumir antes da data
        }
        response = self.create("stock.production.lot", [product_lot])
        return response

    def get_picking_type_id_by_name(
        self, picking_type_name: str, company_id: int, warehouse_id: int | None = None
    ) -> int:
        query = [["name", "=", picking_type_name], ["company_id", "=", company_id]]
        if warehouse_id:
            query.append(["warehouse_id", "=", warehouse_id])
        response = self.search(
            "stock.picking.type",
            [query],
        )
        print(response)
        if not response:
            raise Exception(f"Picking Type '{picking_type_name}' not found")
        elif len(response) > 1:
            raise Exception(
                f"Multiple Picking Types with name '{picking_type_name}' found"
            )
        return response[0]

    def get_picking_type_id_by_id(self, picking_type_id: int) -> list[dict[str, Any]]:
        response = self.read("stock.picking.type", [picking_type_id])
        return response

    def get_picking_type_id_by_warehouse_id(self, warehouse_id: int) -> list[int]:
        response = self.search(
            "stock.picking.type", [[["warehouse_id", "=", warehouse_id]]]
        )
        return response

    def get_picking_id_list_by_sale_order_id(self, sale_order_id: int) -> list[int]:
        response = self.search("stock.picking", [[["sale_id", "=", sale_order_id]]])
        return response

    def get_picking_id_list_by_purchase_order_id(
        self, purchase_order_id: int
    ) -> list[int]:
        response = self.search(
            "stock.picking", [[["purchase_id", "=", purchase_order_id]]]
        )
        return response

    def get_picking_by_id(self, picking_id: int) -> list[dict[str, Any]]:
        response = self.read("stock.picking", [picking_id])
        return response

    def update_picking(self, picking_id: int, data: dict[str, Any]) -> bool:
        response = self.update("stock.picking", picking_id, data)
        return response

    def validate_picking(self, picking_id: int) -> dict[str, Any]:
        response = self.execute_action("stock.picking", "button_validate", picking_id)
        return response

    def confirm_backorder(
        self, sale_order_id: int, picking_id: int, backorder_confirmation_id: int
    ) -> dict[str, Any]:
        extra_context = {
            "active_id": sale_order_id,
            "active_ids": [sale_order_id],
            "button_validate_picking_ids": [picking_id],
            "params": {
                "id": picking_id,
                "active_id": sale_order_id,
                "model": "stock.picking",
                "view_type": "form",
            },
            "contact_display": "partner_address",
            "active_model": "stock.picking",
        }
        response = self.execute_action(
            "stock.backorder.confirmation",
            "process_cancel_backorder",
            backorder_confirmation_id,
            extra_context,
        )
        return response

    def get_stock_picking_move_line_id_by_picking_id(
        self, picking_id: int
    ) -> list[int]:
        response = self.search("stock.move.line", [[["picking_id", "=", picking_id]]])
        return response

    def get_stock_picking_move_line(
        self, stock_move_line_id: int
    ) -> list[dict[str, Any]]:
        response = self.read("stock.move.line", [stock_move_line_id])
        return response

    def confirm_picking(self, picking_data: dict[str, Any]) -> int | list[int]:
        response = self.create("stock.backorder.confirmation", picking_data)
        return response

    def unlink_picking(self, picking_id: int) -> dict[str, Any]:
        response = self.execute_action("stock.picking", "unlink", picking_id)
        return response

    def get_stock_quantity_id_by_any_filter(
        self, filter: list[Any]
    ) -> list[dict[str, Any]]:
        response = self.search("stock.quant", [filter])
        return response

    def get_stock_quantity(self, stock_quantity_id: int) -> list[dict[str, Any]]:
        response = self.read("stock.quant", [stock_quantity_id])
        return response

    def update_stock_quantity(
        self, stock_quantity_id: int, data: dict[str, Any]
    ) -> bool:
        response = self.update("stock.quant", stock_quantity_id, data)
        return response

    def create_stock_quantity(self, data: dict[str, Any]) -> int | list[int]:
        response = self.create("stock.quant", [data])
        return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class SaleOrderModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_sale_order_by_any_filter(self, filter: list[Any]) -> list[int]:
        response = self.search("sale.order", [filter])
        return response

    def get_sale_order_by_state(self, state: list[str]) -> list[int]:
        response = self.search("sale.order", [[["state", "in", state]]])
        return response

    def get_sale_order_by_id(
        self, sale_order_id: int, fields: list[str] | None = None
    ) -> list[dict[str, Any]]:
        response = self.read("sale.order", [sale_order_id], fields)
        return response

    def get_sale_order_line_list_by_sale_order_id(
        self, sale_order_id: int
    ) -> list[int]:
        response = self.search("sale.order.line", [[["order_id", "=", sale_order_id]]])
        return response

    def get_sale_order_line_by_id(
        self, sale_order_line_id: int
    ) -> list[dict[str, Any]]:
        response = self.read("sale.order.line", [sale_order_line_id])
        return response

    def create_sale_order_line(
        self, sale_order_line_data: dict[str, Any]
    ) -> int | list[int]:
        response = self.create("sale.order.line", [sale_order_line_data])
        return response

    def create_sale_order(self, sale_order_data: dict[str, Any]) -> int | list[int]:
        response = self.create("sale.order", [sale_order_data])
        return response

    def confirm_sale_order(self, sale_order_id: int) -> dict[str, Any]:
        response = self.execute_action("sale.order", "action_confirm", sale_order_id)
        return response

    def certify_sale_order(self, sale_order_id: int) -> dict[str, Any]:
        response = self.execute_action("sale.order", "certify", sale_order_id)
        return response

    def create_invoice_from_sale_order(self, sale_order_id: int) -> dict[str, Any]:
        # sale_order_id is used as active_id (context)
        extra_context = {
            "active_id": sale_order_id,
            "active_ids": [sale_order_id],
            "active_model": "sale.order",
        }
        response = self.execute_action(
            "sale.advance.payment.inv", "create", [], extra_context
        )
        return response

    def confirm_invoice_receipt_from_sale_order(
        self, invoice_id: int, sale_order_id: int
    ) -> dict[str, Any]:
        # sale_order_id is used as active_id (context)
        extra_context = {
            "active_id": sale_order_id,
            "active_ids": [sale_order_id],
            "active_model": "sale.order",
            "default_l10n_pt_type": "receipt_invoice",
            "l10n_pt_type": "receipt_invoice",
        }
        response = self.execute_action(
            "sale.advance.payment.inv", "create_invoices", invoice_id, extra_context
        )
        return response

    def create_invoice_receipt_from_sale_order(
        self, sale_order_id: int, data: dict[str, Any]
    ) -> dict[str, Any]:
        # sale_order_id is used as active_id (context)
        extra_context = {
            "active_id": sale_order_id,
            "active_ids": [sale_order_id],
            "active_model": "sale.order",
            "model": "sale.order",
            "default_l10n_pt_type": "receipt_invoice",
            "l10n_pt_type": "receipt_invoice",
            "journal_id": 19,
        }
        response = self.execute_action(
            "sale.advance.payment.inv", "create", data, extra_context
        )
        return response

    def confirm_invoice_from_sale_order(
        self, invoice_id: int, sale_order_id: int
    ) -> dict[str, Any]:
        # sale_order_id is used as active_id (context)
        extra_context = {
            "active_id": sale_order_id,
            "active_ids": [sale_order_id],
            "active_model": "sale.order",
        }
        response = self.execute_action(
            "sale.advance.payment.inv", "create_invoices", invoice_id, extra_context
        )
        return response

    def certify_invoice_from_sale_order(self, sale_order_id: int) -> dict[str, Any]:
        response = self.execute_action("sale.order", "certify", sale_order_id)
        return response

    def get_sale_order_message(self, message_id: int) -> list[dict[str, Any]]:
        response = self.read("mail.message", [message_id])
        return response

    def get_sale_order_attachment(self, attachment_id: int) -> dict[str, Any]:
        response = self.read("ir.attachment", [attachment_id])
        return response[0]

    def confirm_invoice_by_button_action_by_invoice_id(
        self, invoice_id: int
    ) -> dict[str, Any]:
        response = self.execute_action("account.move", "action_post", invoice_id)
        return response

    def get_invoice_id_from_sale_order(self, sale_order_id: int) -> int:
        response = self.execute_action(
            "sale.order", "action_view_invoice", sale_order_id
        )
        return response["res_id"]

    def invoice_force_creation(self, invoice_id: int) -> bool:
        response = self.update("account.move", invoice_id, {"force_open": True})
        return response

    def invoice_insert_footer_notes(self, invoice_id: int, notes: str) -> bool:
        response = self.update("account.move", invoice_id, {"footer_notes": notes})
        return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class AccountInvoiceModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def invoice_send_email(self, invoice_id: int, mail_id: int) -> dict[str, Any]:
        response = self.execute_action(
            "account.invoice.send", "send_and_print_action", mail_id
        )
        return response

    def create_invoice_send_email(
        self,
        invoice_id: int,
        template_id: int,
        partner_id: int,
        partner_ids: list[int],
        attachment_id: int,
        mail_server_id: int,
        composition_mode: str,
        subject: str,
        body: str,
    ) -> dict[str, Any]:
        # onchange or call
        extra_context = {
            "active_id": invoice_id,
            "active_ids": [invoice_id],
            "active_model": "account.move",
            "custom_layout": "mail.mail_notification_paynow",
            "default_model": "account.move",
            "default_res_id": invoice_id,
            "default_res_model": "account.move",
            "default_template_id": template_id,
            "default_use_template": True,
            "force_email": True,
            "mark_invoice_as_sent": True,
            "model_description": "Fatura",
        }
        params = [
            {
                "composition_mode": composition_mode,
                "invoice_ids": [[6, False, [invoice_id]]],
                "email_from": '"Rafael Galleani" <rafegal@gmail.com>',
                "mail_server_id": mail_server_id,
                "is_print": False,
                "snailmail_is_letter": False,
                "partner_id": partner_id,
                "is_email": True,
                "partner_ids": partner_ids,
                "subject": subject,
                "body": body,
                "attachment_ids": [[6, False, [attachment_id]]],
                "template_id": template_id,
            }
        ]
        response = self.execute_action(
            "account.invoice.send", "create", params[0], extra_context=extra_context
        )
        return response

    def get_default_invoice_template(self, sale_order_id: int) -> dict[str, Any]:
        response = self.execute_action(
            "account.move", "action_invoice_sent", sale_order_id
        )
        return response

    def get_template_email(
        self, invoice_id: int, default_template_id: int
    ) -> dict[str, Any]:
        # onchange or call
        extra_context = {
            "active_id": invoice_id,
            "active_ids": [invoice_id],
            "active_model": "account.move",
            "custom_layout": "mail.mail_notification_paynow",
            "default_model": "account.move",
            "default_res_id": invoice_id,
            "default_res_model": "account.move",
            "default_template_id": default_template_id,
            "default_use_template": True,
            "force_email": True,
            "mark_invoice_as_sent": True,
            "model_description": "Fatura",
        }
        params = {
            "move_types": "",
            "composition_mode": "1",
            "invoice_ids": "1",
            "email_from": "1",
            "mail_server_id": "1",
            "is_print": "",
            "invalid_addresses": "",
            "snailmail_is_letter": "1",
            "snailmail_cost": "",
            "partner_id": "",
            "is_email": "1",
            "invoice_without_email": "",
            "partner_ids": "1",
            "subject": "1",
            "body": "1",
            "attachment_ids": "1",
            "template_id": "1",
        }
        response = self.execute_action_onchange(
            "account.invoice.send", params, extra_context=extra_context
        )
        return response



from __future__ import annotations
from ._integration import OdooIntegration


class PaymentTermModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_payment_term_id_by_name(self, payment_term_name: str) -> int:
        response = self.search(
            "account.payment.term", [[["name", "=", payment_term_name]]]
        )
        if response:
            return response[0]
        raise Exception(f"Payment term name '{payment_term_name}' not found")



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class AccountTaxModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_tax_id_by_name(self, tax_name: str, company_id: int | None = None) -> int:
        if company_id:
            response = self.search(
                "account.tax",
                [[["name", "=", tax_name], ["company_id", "=", company_id]]],
            )
        else:
            response = self.search("account.tax", [[["name", "=", tax_name]]])
        if response:
            return response[0]
        raise Exception(f"Tax '{tax_name}' not found")

    def get_tax_by_id(
        self, tax_id: int, fields: list[str] | None = None
    ) -> list[dict[str, Any]]:
        response = self.read("account.tax", [tax_id], fields)
        return response



from __future__ import annotations
from typing import Any
from environs import Env
import xmlrpc.client

env = Env()
env.read_env()


class OdooGenericIntegration:
    def __init__(
        self,
        odoo_url: str | None = None,
        odoo_db: str | None = None,
        odoo_username: str | None = None,
        odoo_password: str | None = None,
        odoo_language: str | None = None,
    ) -> None:
        self._url = odoo_url or env.str("ODOO_URL")
        self._db = odoo_db or env.str("ODOO_DB")
        self._username = odoo_username or env.str("ODOO_USERNAME")
        self._password = odoo_password or env.str("ODOO_PASSWORD")
        self._language = odoo_language or env.str("ODOO_LANGUAGE", default="es_ES")
        self._common = xmlrpc.client.ServerProxy("{}/xmlrpc/2/common".format(self._url))
        self._uid = self._common.authenticate(
            self._db, self._username, self._password, {}
        )
        self._models = xmlrpc.client.ServerProxy("{}/xmlrpc/2/object".format(self._url))

    def execute_kw(
        self,
        model: str,
        method: str,
        args: list[Any],
        kwargs: dict[str, Any] | None = None,
    ) -> list[int]:
        default_kwargs = {"context": {"lang": self._language}}
        if kwargs:
            default_kwargs.update(kwargs)
        response = self._models.execute_kw(
            self._db,
            self._uid,
            self._password,
            model,
            method,
            args,
            default_kwargs,
        )
        return response

    def search(
        self,
        model: str,
        search_params: list[Any],
        kwargs_data: dict[str, Any] | None = None,
    ) -> list[int]:
        response = self.execute_kw(
            model=model,
            method="search",
            args=[search_params],
            kwargs=kwargs_data,
        )
        return response

    def search_read(
        self,
        model: str,
        search_params: list[Any],
        kwargs_data: dict[str, Any] | None = None,
    ) -> list[int]:
        response = self.execute_kw(
            model=model,
            method="search_read",
            args=[search_params],
            kwargs=kwargs_data,
        )
        return response

    def search_count(
        self,
        model: str,
        search_params: list[Any],
        kwargs_data: dict[str, Any] | None = None,
    ) -> int:
        response = self.execute_kw(
            model=model,
            method="search_count",
            args=[search_params],
            kwargs=kwargs_data,
        )
        return response

    def read(
        self,
        model: str,
        ids: list[int | str],
        kwargs_data: dict[str, Any] | None = None,
    ) -> list[dict[str, Any]]:
        response = self.execute_kw(
            model=model,
            method="read",
            args=[ids],
            kwargs=kwargs_data,
        )
        return response

    def create(
        self,
        model: str,
        data: dict[str, Any],
        kwargs_data: dict[str, Any] | None = None,
    ) -> list[dict[str, Any]]:
        response = self.execute_kw(
            model=model,
            method="create",
            args=[data],
            kwargs=kwargs_data,
        )
        return response

    def bulk_create(
        self,
        model: str,
        data: list[dict[str, Any]],
        kwargs_data: dict[str, Any] | None = None,
    ) -> list[dict[str, Any]]:
        response = self.execute_kw(
            model=model,
            method="create",
            args=data,
            kwargs=kwargs_data,
        )
        return response

    def update(
        self,
        model: str,
        object_id: int | str,
        data: list[dict[str, Any]],
        kwargs_data: dict[str, Any] | None = None,
    ) -> list[dict[str, Any]]:
        response = self.execute_kw(
            model=model,
            method="write",
            args=[[object_id], data],
            kwargs=kwargs_data,
        )
        return response

    def bulk_update(
        self,
        model: str,
        records: list[dict[str, Any]],
        kwargs_data: dict[str, Any] | None = None,
    ) -> list[dict[str, Any]]:
        response = self.execute_kw(
            model=model,
            method="write",
            args=records,
            kwargs=kwargs_data,
        )
        return response

    def call_method(
        self,
        model: str,
        call_method: str,
        args: list[any],
        kwargs: dict[str, Any] | None = None,
    ) -> dict[str, Any]:
        response = self.execute_kw(
            model=model,
            method=call_method,
            args=args,
            kwargs=kwargs,
        )
        return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration
from .exceptions import ProductNotFoundError


class ProductModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_product_by_id(self, product_id: int) -> list[dict[str, Any]]:
        response = self.read("product.product", [product_id])
        return response

    def get_product_id_by_reference(self, reference_id: str) -> int:
        response = self.search(
            "product.product", [[["default_code", "=", reference_id]]]
        )
        if response:
            return response[0]
        raise ProductNotFoundError(f"Product '{reference_id}' not found")



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class AccountMoveModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def confirm_invoice_purchase_order(self, invoice_id: int) -> dict[str, Any]:
        response = self.execute_action("account.move", "action_post", invoice_id)
        return response

    def invoice_insert_footer_notes(self, invoice_id: int, notes: str) -> bool:
        response = self.update("account.move", invoice_id, {"footer_notes": notes})
        return response

    def invoice_insert_product_line(
        self, invoice_id: int, data: dict[str, Any]
    ) -> bool:
        response = self.update("account.move", invoice_id, data)
        return response

    def get_invoice_by_id(self, invoice_id: int) -> list[dict[str, Any]]:
        response = self.read("account.move", [invoice_id])
        return response

    def get_invoice_line_by_id(self, account_move_line_id: int) -> list[dict[str, Any]]:
        response = self.read("account.move.line", [account_move_line_id])
        return response

    def get_invoice_lines_by_any_filter(self, filter: list[Any]) -> list[int]:
        response = self.search("account.move.line", [filter])
        return response

    def update_invoice_line(self, invoice_id: int, data: dict[str, Any]) -> bool:
        response = self.update("account.move", invoice_id, data)
        return response

    def get_account_move_line_by_id(
        self, account_move_line_id: int
    ) -> list[dict[str, Any]]:
        response = self.read("account.move.line", [account_move_line_id])
        return response

    def get_account_move_by_id(
        self, account_move_id: int, fields: list[str] | None = None
    ) -> list[dict[str, Any]]:
        response = self.read("account.move", [account_move_id], fields=fields)
        return response

    def create_account_move(self, account_move_data: dict[str, Any]) -> int | list[int]:
        response = self.create("account.move", [account_move_data])
        return response

    def create_account_move_line(
        self, account_move_line_data: dict[str, Any]
    ) -> int | list[int]:
        response = self.create("account.move.line", [account_move_line_data])
        return response

    def update_account_move_line(
        self, account_move_line_id: int, data: dict[str, Any]
    ) -> bool:
        response = self.update("account.move.line", account_move_line_id, data)
        return response

    def update_account_move(self, account_move_id: int, data: dict[str, Any]) -> bool:
        response = self.update("account.move", account_move_id, data)
        return response

    def confirm_account_move(self, account_move_id: int) -> dict[str, Any]:
        response = self.execute_action("account.move", "action_post", account_move_id)
        return response

    def create_account_payment_register(
        self, data: dict[str, Any], context: dict[str, Any] | None = None
    ) -> int | list[int]:
        response = self.create("account.payment.register", [data], context)
        return response

    def execute_action_create_payments(
        self, account_payment_register_id: int, context: dict[str, Any] | None = None
    ) -> dict[str, Any]:
        response = self.execute_action(
            "account.payment.register",
            "action_create_payments",
            account_payment_register_id,
            context,
        )
        return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class IRAttachmentModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_ir_attachment(self, attachment_id: int) -> list[dict[str, Any]]:
        response = self.read("ir.attachment", [attachment_id])
        return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class AccountJournalModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_journal_id_by_name(
        self, journal_name: str, company_id: int | None = None
    ) -> int:
        if company_id:
            response = self.search(
                "account.journal",
                [[["name", "=", journal_name], ["company_id", "=", company_id]]],
            )
        else:
            response = self.search("account.journal", [[["name", "=", journal_name]]])
        if response:
            return response[0]
        raise Exception(f"Journal '{journal_name}' not found")

    def get_journal_by_id(
        self, journal_id: int, fields: list[str] | None = None
    ) -> list[dict[str, Any]]:
        response = self.read("account.journal", [journal_id], fields)
        return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class AccountAccountModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_account_by_id(self, account_id: int) -> list[int]:
        response = self.read("account.account", [account_id])
        return response

    def get_account_by_any_filter(self, filter: list[Any]) -> list[int]:
        response = self.search("account.account", [filter])
        return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class PurchaseOrderModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_purchase_order_by_any_filter(self, filter: list[Any]) -> list[int]:
        response = self.search("purchase.order", [filter])
        return response

    def get_purchase_order_by_state(self, state: list[str]) -> list[int]:
        response = self.search("purchase.order", [[["state", "in", state]]])
        return response

    def get_purchase_order_by_id(
        self, purchase_order_id: int, fields: list[str] | None = None
    ) -> list[dict[str, Any]]:
        response = self.read("purchase.order", [purchase_order_id], fields)
        return response

    def get_purchase_order_line_list_by_purchase_order_id(
        self, purchase_order_id: int
    ) -> list[int]:
        response = self.search(
            "purchase.order.line", [[["order_id", "=", purchase_order_id]]]
        )
        return response

    def get_purchase_order_line_by_id(
        self, purchase_order_line_id: int
    ) -> list[dict[str, Any]]:
        response = self.read("purchase.order.line", [purchase_order_line_id])
        return response

    def create_purchase_order_line(
        self, purchase_order_line_data: dict[str, Any]
    ) -> int | list[int]:
        response = self.create("purchase.order.line", [purchase_order_line_data])
        return response

    def create_purchase_order(
        self, purchase_order_data: dict[str, Any]
    ) -> int | list[int]:
        response = self.create("purchase.order", [purchase_order_data])
        return response

    def create_invoice_from_purchase_order(
        self, purchcase_order_id: int
    ) -> dict[str, Any]:
        response = self.execute_action(
            "purchase.order", "action_create_invoice", purchcase_order_id
        )
        return response

    def confirm_invoice_purchase_order(self, invoice_id: int) -> dict[str, Any]:
        response = self.execute_action("account.move", "action_post", invoice_id)
        return response

    def invoice_insert_footer_notes(self, invoice_id: int, notes: str) -> bool:
        response = self.update("account.move", invoice_id, {"footer_notes": notes})
        return response

    """
    method: call
    method: action_post
    model_ account.move

    """

    # def confirm_purchase_order(self, purchase_order_id):
    #     response = self.execute_action("purchase.order", "action_confirm", purchase_order_id)
    #     return response

    # def certify_purchase_order(self, purchase_order_id):
    #     response = self.execute_action("purchase.order", "certify", purchase_order_id)
    #     return response



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class CountryModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_country_id_by_code(self, country_code: str) -> int:
        response = self.search("res.country", [[["code", "=", country_code]]])
        if response:
            return response[0]
        raise Exception(f"Country '{country_code}' not found")

    def get_country_id_by_name(self, country_name: str) -> int:
        response = self.search("res.country", [[["name", "=", country_name]]])
        if response:
            return response[0]
        raise Exception(f"Country '{country_name}' not found")

    def get_country_state_by_id(self, state_id: int) -> dict[str, Any]:
        response = self.read("res.country.state", [state_id])
        if response:
            return response[0]
        raise Exception(f"State '{state_id}' not found")

    def get_country_state_id_by_name(self, state_name: str) -> int:
        response = self.search("res.country.state", [[["name", "=", state_name]]])
        if response:
            return response[0]
        raise Exception(f"State '{state_name}' not found")



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

setup(
    name="odoo-py",
    version="0.0.19",
    author="Rafael Galleani",
    description="Interface to call Odoo API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/IT-NOSSA-FARMACIA/odoo-py",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.9",
    install_requires=requirements,
    dependency_links=[],
)



from __future__ import annotations
from typing import Any
from ._integration import OdooIntegration


class MailMessageModel(OdooIntegration):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)

    def get_mail_message(self, message_id: int) -> list[dict[str, Any]]:
        response = self.read("mail.message", [message_id])
        return response
